# Generated by Django 2.1 on 2018-10-06 20:57

import edc_sites.models
from django.db import migrations

import edc_registration.models.managers


class Migration(migrations.Migration):

    dependencies = [("edc_registration", "0014_auto_20180814_2017")]

    operations = [
        migrations.AlterModelManagers(
            name="registeredsubject",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                (
                    "objects",
                    edc_registration.models.managers.RegisteredSubjectManager(),
                ),
            ],
        )
    ]
