# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2016-12-05 08:04
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_registration", "0002_auto_20161127_2226")]

    operations = [
        migrations.AddField(
            model_name="registeredsubject",
            name="subject_type",
            field=models.CharField(blank=True, max_length=25, null=True),
        )
    ]
