# Generated by Django 2.2.3 on 2019-09-17 18:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_registration", "0017_auto_20190305_0123")]

    operations = [
        migrations.AddField(
            model_name="historicalregisteredsubject",
            name="randomization_list_model",
            field=models.CharField(max_length=150, null=True),
        ),
        migrations.AddField(
            model_name="registeredsubject",
            name="randomization_list_model",
            field=models.CharField(max_length=150, null=True),
        ),
    ]
